@@ -1,7 +1,7 @@
 #
 #    Uncomplicated VM Builder
-#    Copyright (C) 2007-2009 Canonical Ltd.
-#    
+#    Copyright (C) 2007-2010 Canonical Ltd.
+#
 #    See AUTHORS for list of contributors
 #
 #    This program is free software: you can redistribute it and/or modify
@@ -18,51 +18,64 @@
 #
 #    Virtual disk management
 
+import fcntl
 import logging
+import os
 import os.path
 import re
 import stat
 import string
-import tempfile
-import VMBuilder
+import time
 from   VMBuilder.util      import run_cmd 
 from   VMBuilder.exception import VMBuilderUserError, VMBuilderException
+from   struct              import unpack
 
 TYPE_EXT2 = 0
 TYPE_EXT3 = 1
 TYPE_XFS = 2
 TYPE_SWAP = 3
+TYPE_EXT4 = 4
 
 class Disk(object):
-    def __init__(self, vm, size='5G', preallocated=False, filename=None):
-        """
-        @type  size: string or number
-        @param size: The size of the disk image (passed to L{parse_size})
+    """
+    Virtual disk.
 
-        @type  preallocated: boolean
-        @param preallocated: if True, the disk image already exists and will not be created (useful for raw devices)
-
-        @type  filename: string
-        @param filename: force a certain filename or to give the name of the preallocated disk image
-        """
-
-        # We need this for "introspection"
+    @type  vm: Hypervisor
+    @param vm: The Hypervisor to which the disk belongs
+    @type  filename: string
+    @param filename: filename of the disk image
+    @type  size: string or number
+    @param size: The size of the disk image to create (passed to
+        L{parse_size}). If specified and filename already exists,
+        L{VMBuilderUserError} will be raised. Otherwise, a disk image of
+        this size will be created once L{create}() is called.
+    """
+    
+    def __init__(self, vm, filename, size=None):
         self.vm = vm
+        "The hypervisor to which the disk belongs."
 
-        # Perhaps this should be the frontend's responsibility?
-        self.size = parse_size(size)
-
-        self.preallocated = preallocated
-
-        # If filename isn't given, make one up
-        if filename:
-            self.filename = filename
-        else:
-            if self.preallocated:
-                raise VMBuilderUserError('Preallocated was set, but no filename given')
-            self.filename = 'disk%d.img' % len(self.vm.disks)
+        self.filename = filename
+        "The filename of the disk image."
 
         self.partitions = []
+        "The list of partitions on the disk. Is kept in order by L{add_part}."
+
+        self.preallocated = False
+        "Whether the file existed already (True if it did, False if we had to create it)."
+
+        self.size = 0
+        "The size of the disk. For preallocated disks, this is detected."
+
+        if not os.path.exists(self.filename):
+            if not size:
+                raise VMBuilderUserError('%s does not exist, but no size was given.' % (self.filename))
+            self.size = parse_size(size)
+        else:
+            if size:
+                raise VMBuilderUserError('%s exists, but size was given.' % (self.filename))
+            self.preallocated = True
+            self.size = detect_size(self.filename)
 
     def devletters(self):
         """
@@ -73,23 +86,25 @@ class Disk(object):
 
         return index_to_devname(self.vm.disks.index(self))
 
-    def create(self, directory):
+    def create(self):
         """
-        Creates the disk image (unless preallocated), partitions it, creates the partition mapping devices and mkfs's the partitions
+        Creates the disk image (if it doesn't already exist).
 
-        @type  directory: string
-        @param directory: If set, the disk image is created in this directory
+        Once this method returns succesfully, L{filename} can be
+        expected to points to point to whatever holds the virtual disk
+        (be it a file, partition, logical volume, etc.).
         """
-
-        if not self.preallocated:
-            if directory:
-                self.filename = '%s/%s' % (directory, self.filename)
-            logging.info('Creating disk image: %s' % self.filename)
+        if not os.path.exists(self.filename):
+            logging.info('Creating disk image: "%s" of size: %dMB' % (self.filename, self.size))
             run_cmd(qemu_img_path(), 'create', '-f', 'raw', self.filename, '%dM' % self.size)
-            os.chmod(self.filename, stat.S_IRUSR | stat.S_IWUSR)
 
-        # From here, we assume that self.filename refers to whatever holds the disk image,
-        # be it a file, a partition, logical volume, actual disk..
+    def partition(self):
+        """
+        Partitions the disk image. First adds a partition table and then
+        adds the individual partitions.
+
+        Should only be called once and only after you've added all partitions.
+        """
 
         logging.info('Adding partition table to disk image: %s' % self.filename)
         run_cmd('parted', '--script', self.filename, 'mklabel', 'msdos')
@@ -98,6 +113,15 @@ class Disk(object):
         for part in self.partitions:
             part.create(self)
 
+    def map_partitions(self):
+        """
+        Create loop devices corresponding to the partitions.
+
+        Once this has returned succesfully, each partition's map device
+        is set as its L{filename<Disk.Partition.filename>} attribute.
+
+        Call this after L{partition}.
+        """
         logging.info('Creating loop devices corresponding to the created partitions')
         self.vm.add_clean_cb(lambda : self.unmap(ignore_fail=True))
         kpartx_output = run_cmd('kpartx', '-av', self.filename)
@@ -113,12 +137,12 @@ class Disk(object):
         for line in parts:
             mapdevs.append(line.split(' ')[2])
         for (part, mapdev) in zip(self.partitions, mapdevs):
-            part.mapdev = '/dev/mapper/%s' % mapdev
+            part.set_filename('/dev/mapper/%s' % mapdev)
 
-        # At this point, all partitions are created and their mapping device has been
-        # created and set as .mapdev
-
-        # Adds a filesystem to the partition
+    def mkfs(self):
+        """
+        Creates the partitions' filesystems
+        """
         logging.info("Creating file systems")
         for part in self.partitions:
             part.mkfs()
@@ -133,17 +157,37 @@ class Disk(object):
     def get_index(self):
         """
         @rtype:  number
-        @return: index of the disk (starting from 0)
+        @return: index of the disk (starting from 0 for the hypervisor's first disk)
         """
         return self.vm.disks.index(self)
 
     def unmap(self, ignore_fail=False):
         """
         Destroy all mapping devices
+
+        Unsets L{Partition}s' and L{Filesystem}s' filename attribute
         """
+        # first sleep to give the loopback devices a chance to settle down
+        time.sleep(3)
+
+        tries = 0
+        max_tries = 3
+        while tries < max_tries:
+            try:
+                run_cmd('kpartx', '-d', self.filename, ignore_fail=False)
+                break
+            except:
+                pass
+            tries += 1
+            time.sleep(3)
+
+            if tries >= max_tries:
+                # try it one last time
+                logging.info("Could not unmount '%s' after '%d' attempts. Final attempt" % (self.filename, tries))
         run_cmd('kpartx', '-d', self.filename, ignore_fail=ignore_fail)
+
         for part in self.partitions:
-            self.mapdev = None
+            part.set_filename(None)
 
     def add_part(self, begin, length, type, mntpnt):
         """
@@ -158,16 +202,15 @@ class Disk(object):
         @type  mntpnt: string
         @param mntpnt: Intended mountpoint inside the guest of the new partition
         """
+        length = parse_size(length)
         end = begin+length-1
-        logging.debug("add_part - begin %d, length %d, end %d" % (begin, length, end))
+        logging.debug("add_part - begin %d, length %d, end %d, type %s, mntpnt %s" % (begin, length, end, type, mntpnt))
         for part in self.partitions:
             if (begin >= part.begin and begin <= part.end) or \
                 (end >= part.begin and end <= part.end):
-                raise Exception('Partitions are overlapping')
-            if begin > end:
-                raise Exception('Partition\'s last block is before its first')
-            if begin < 0 or end > self.size:
-                raise Exception('Partition is out of bounds. start=%d, end=%d, disksize=%d' % (begin,end,self.size))
+                raise VMBuilderUserError('Partitions are overlapping')
+        if begin < 0 or end > self.size:
+            raise VMBuilderUserError('Partition is out of bounds. start=%d, end=%d, disksize=%d' % (begin,end,self.size))
         part = self.Partition(disk=self, begin=begin, end=end, type=str_to_type(type), mntpnt=mntpnt)
         self.partitions.append(part)
 
@@ -177,7 +220,7 @@ class Disk(object):
     def convert(self, destdir, format):
         """
         Convert the disk image
-        
+
         @type  destdir: string
         @param destdir: Target location of converted disk image
         @type  format: string
@@ -206,18 +249,36 @@ class Disk(object):
     class Partition(object):
         def __init__(self, disk, begin, end, type, mntpnt):
             self.disk = disk
+            "The disk on which this Partition resides."
+
             self.begin = begin
+            "The start of the partition"
+
             self.end = end
+            "The end of the partition"
+
             self.type = type
+            "The partition type"
+
             self.mntpnt = mntpnt
-            self.mapdev = None
+            "The destined mount point"
+
+            self.filename = None
+            "The filename of this partition (the map device)"
+
+            self.fs = Filesystem(vm=self.disk.vm, type=self.type, mntpnt=self.mntpnt)
+            "The enclosed filesystem"
+
+        def set_filename(self, filename):
+            self.filename = filename
+            self.fs.filename = filename
 
         def parted_fstype(self):
             """
             @rtype: string
             @return: the filesystem type of the partition suitable for passing to parted
             """
-            return { TYPE_EXT2: 'ext2', TYPE_EXT3: 'ext2', TYPE_XFS: 'ext2', TYPE_SWAP: 'linux-swap(new)' }[self.type]
+            return { TYPE_EXT2: 'ext2', TYPE_EXT3: 'ext2', TYPE_EXT4: 'ext2', TYPE_XFS: 'ext2', TYPE_SWAP: 'linux-swap(new)' }[self.type]
 
         def create(self, disk):
             """Adds partition to the disk image (does not mkfs or anything like that)"""
@@ -226,9 +287,6 @@ class Disk(object):
 
         def mkfs(self):
             """Adds Filesystem object"""
-            if not self.mapdev:
-                raise Exception('We can\'t mkfs before we have a mapper device')
-            self.fs = Filesystem(self.disk.vm, preallocated=True, filename=self.mapdev, type=self.type, mntpnt=self.mntpnt)
             self.fs.mkfs()
 
         def get_grub_id(self):
@@ -244,29 +302,34 @@ class Disk(object):
             """Index of the disk (starting from 0)"""
             return self.disk.partitions.index(self)
 
+        def set_type(self, type):
+            try:
+                if int(type) == type:
+                    self.type = type
+                else:
+                    self.type = str_to_type(type)
+            except ValueError:
+                self.type = str_to_type(type)
+
 class Filesystem(object):
-    def __init__(self, vm, size=0, preallocated=False, type=None, mntpnt=None, filename=None, devletter='a', device='', dummy=False):
+    def __init__(self, vm=None, size=0, type=None, mntpnt=None, filename=None, devletter='a', device='', dummy=False):
         self.vm = vm
         self.filename = filename
         self.size = parse_size(size)
-        self.preallocated = preallocated
         self.devletter = devletter
         self.device = device
         self.dummy = dummy
-           
-        try:
-            if int(type) == type:
-                self.type = type
-            else:
-                self.type = str_to_type(type)
-        except ValueError, e:
-            self.type = str_to_type(type)
+
+        self.set_type(type)
 
         self.mntpnt = mntpnt
 
+        self.preallocated = False
+        "Whether the file existed already (True if it did, False if we had to create it)."
+
     def create(self):
         logging.info('Creating filesystem: %s, size: %d, dummy: %s' % (self.mntpnt, self.size, repr(self.dummy)))
-        if not self.preallocated:
+        if not os.path.exists(self.filename):
             logging.info('Not preallocated, so we create it.')
             if not self.filename:
                 if self.mntpnt:
@@ -288,32 +351,36 @@ class Filesystem(object):
         self.mkfs()
 
     def mkfs(self):
+        if not self.filename:
+            raise VMBuilderException('We can\'t mkfs if filename is not set. Did you forget to call .create()?')
         if not self.dummy:
             cmd = self.mkfs_fstype() + [self.filename]
             run_cmd(*cmd)
+            # Let udev have a chance to extract the UUID for us
+            run_cmd('udevadm', 'settle')
             if os.path.exists("/sbin/vol_id"):
                 self.uuid = run_cmd('vol_id', '--uuid', self.filename).rstrip()
             elif os.path.exists("/sbin/blkid"):
-                self.uuid = run_cmd('blkid', '-sUUID', '-ovalue', self.filename).rstrip()
+                self.uuid = run_cmd('blkid', '-p', '-sUUID', '-ovalue', self.filename).rstrip()
 
     def mkfs_fstype(self):
-        if self.vm.suite in ['dapper', 'edgy', 'feisty', 'gutsy']:
-            logging.debug('%s: 128 bit inode' % self.vm.suite)
-            return { TYPE_EXT2: ['mkfs.ext2', '-F'], TYPE_EXT3: ['mkfs.ext3', '-I 128', '-F'], TYPE_XFS: ['mkfs.xfs'], TYPE_SWAP: ['mkswap'] }[self.type]
-        else:
-            logging.debug('%s: 256 bit inode' % self.vm.suite)
-            return { TYPE_EXT2: ['mkfs.ext2', '-F'], TYPE_EXT3: ['mkfs.ext3', '-F'], TYPE_XFS: ['mkfs.xfs'], TYPE_SWAP: ['mkswap'] }[self.type]
+        map = { TYPE_EXT2: ['mkfs.ext2', '-F'], TYPE_EXT3: ['mkfs.ext3', '-F'], TYPE_EXT4: ['mkfs.ext4', '-F'], TYPE_XFS: ['mkfs.xfs'], TYPE_SWAP: ['mkswap'] }
+
+        if not self.vm.distro.has_256_bit_inode_ext3_support():
+            map[TYPE_EXT3] = ['mkfs.ext3', '-I 128', '-F']
+
+        return map[self.type]
 
     def fstab_fstype(self):
-        return { TYPE_EXT2: 'ext2', TYPE_EXT3: 'ext3', TYPE_XFS: 'xfs', TYPE_SWAP: 'swap' }[self.type]
+        return { TYPE_EXT2: 'ext2', TYPE_EXT3: 'ext3', TYPE_EXT4: 'ext4', TYPE_XFS: 'xfs', TYPE_SWAP: 'swap' }[self.type]
 
     def fstab_options(self):
         return 'defaults'
 
-    def mount(self):
+    def mount(self, rootmnt):
         if (self.type != TYPE_SWAP) and not self.dummy:
             logging.debug('Mounting %s', self.mntpnt) 
-            self.mntpath = '%s%s' % (self.vm.rootmnt, self.mntpnt)
+            self.mntpath = '%s%s' % (rootmnt, self.mntpnt)
             if not os.path.exists(self.mntpath):
                 os.makedirs(self.mntpath)
             run_cmd('mount', '-o', 'loop', self.filename, self.mntpath)
@@ -344,17 +411,26 @@ class Filesystem(object):
     def get_index(self):
         """Index of the disk (starting from 0)"""
         return self.vm.filesystems.index(self)
-                
+
+    def set_type(self, type):
+        try:
+            if int(type) == type:
+                self.type = type
+            else:
+                self.type = str_to_type(type)
+        except ValueError:
+            self.type = str_to_type(type)
+
 def parse_size(size_str):
     """Takes a size like qemu-img would accept it and returns the size in MB"""
     try:
         return int(size_str)
-    except ValueError, e:
+    except ValueError:
         pass
 
     try:
         num = int(size_str[:-1])
-    except ValueError, e:
+    except ValueError:
         raise VMBuilderUserError("Invalid size: %s" % size_str)
 
     if size_str[-1:] == 'g' or size_str[-1:] == 'G':
@@ -366,6 +442,7 @@ def parse_size(size_str):
 
 str_to_type_map = { 'ext2': TYPE_EXT2,
                  'ext3': TYPE_EXT3,
+                 'ext4': TYPE_EXT4,
                  'xfs': TYPE_XFS,
                  'swap': TYPE_SWAP,
                  'linux-swap': TYPE_SWAP }
@@ -373,9 +450,13 @@ str_to_type_map = { 'ext2': TYPE_EXT2,
 def str_to_type(type):
     try:
         return str_to_type_map[type]
-    except KeyError, e:
+    except KeyError:
         raise Exception('Unknown partition type: %s' % type)
         
+def rootpart(disks):
+    """Returns the partition which contains the root dir"""
+    return path_to_partition(disks, '/')
+
 def bootpart(disks):
     """Returns the partition which contains /boot"""
     return path_to_partition(disks, '/boot/foo')
@@ -424,7 +505,21 @@ def devname_to_index_rec(devname):
 def index_to_devname(index, suffix=''):
     if index < 0:
         return suffix
-    return suffix + index_to_devname(index / 26 -1, string.ascii_lowercase[index % 26])
+    return index_to_devname(index / 26 -1, string.ascii_lowercase[index % 26]) + suffix
+
+def detect_size(filename):
+    st = os.stat(filename)
+    if stat.S_ISREG(st.st_mode):
+        return st.st_size / 1024*1024
+    elif stat.S_ISBLK(st.st_mode): 
+        # I really wish someone would make these available in Python
+        BLKGETSIZE64 = 2148012658
+        fp = open(filename, 'r')
+        fd = fp.fileno()
+        s = fcntl.ioctl(fd, BLKGETSIZE64, ' '*8)
+        return unpack('L', s)[0] / 1024*1024
+
+    raise VMBuilderException('No idea how to find the size of %s' % filename)
 
 def qemu_img_path():
     exes = ['kvm-img', 'qemu-img']
