@@ -34,10 +34,3 @@ console.setLevel(logging.INFO)
 console.setFormatter(logging.Formatter(format))
 logging.getLogger('').addHandler(console)
 
-def set_verbosity(option, opt_str, value, parser):
-    if opt_str == '--debug':
-        console.setLevel(logging.DEBUG)
-    elif opt_str == '--verbose':
-        console.setLevel(logging.INFO)
-    elif opt_str == '--quiet':
-        console.setLevel(logging.CRITICAL)
